/*
  "spl_cosmic.cc"
  Measurement of the effective speed of light in S-2S TOF detector.
  This analysis is for data with cosmic ray.
  
  Modified by Toshi Gogami , 13Nov2014
*/


//void light(){
void spl_cosmic(){
  // ======== General conditions =======
  gROOT->SetStyle("Plain");
  //gStyle->SetOptStat(1111111);
  //gStyle->SetOptStat(1);
  gStyle->SetOptStat(0);
  
  // ======== Open ROOT file ===========
  TFile* f1 = new TFile("TOFcenter_run95-run101.root"); 
  TFile* f2 = new TFile("TOF_moved+11.5cm_run57-run93.root");  
  TTree* t1 = (TTree*)f1->Get("tree");
  TTree* t2 = (TTree*)f2->Get("tree");
  
  // ====== Create histograms =========
  const double timeoffset = 7.24; // Offset parameter
  int bin1 = 200;
  Double_t min1 = -10.0,max1 = 10.0;
  TH1D* h_center = new TH1D("h_center","",bin1,min1,max1); 
  h_center->SetFillColor(4);
  h_center->SetFillStyle(3001);
  h_center->GetXaxis()->SetTitle("[ns]");
  h_center->GetYaxis()->SetTitle("[Counts / 0.1 ns]");
  h_center->GetXaxis()->SetRangeUser(-5.0,5.0);
  
  TH1D* h_center_scaled = (TH1D*)h_center->Clone("h_center_scaled");
  TH1D* h_plus = (TH1D*)h_center->Clone("h_plus");
  TH1D* h_plus_scaled = (TH1D*)h_plus->Clone("h_plus_scaled");
  
  // ====== Define variables =================
  Double_t time1_1=0.0, time2_1=0.0, time3_1=0.0, time4_1=0.0;
  Int_t adc1_1=0.0, adc2_1=0.0, adc3_1=0.0, adc4_1=0.0;
  
  Double_t time1_2=0.0, time2_2=0.0, time3_2=0.0, time4_2=0.0;
  Int_t adc1_2=0.0, adc2_2=0.0, adc3_2=0.0, adc4_2=0.0;
  
  Double_t time1_3=0.0, time2_3=0.0, time3_3=0.0, time4_3=0.0;
  Int_t adc1_3=0.0, adc2_3=0.0, adc3_3=0.0, adc4_3=0.0;
  
  bool cut = false; // cut conditions 
  Double_t max1=0.0, max2=0.0, max3=0.0;

  // ======== LOOP for "f1" ==================
  t1->SetBranchAddress("time1",  &time1_1); // time = tdc * 0.05 [ns]
  t1->SetBranchAddress("time2",  &time2_1); // time = tdc * 0.05 [ns]
  t1->SetBranchAddress("time3",  &time3_1); // time = tdc * 0.05 [ns]
  t1->SetBranchAddress("time4",  &time4_1); // time = tdc * 0.05 [ns]
  t1->SetBranchAddress("adc1",   &adc1_1); // ADC
  t1->SetBranchAddress("adc2",   &adc2_1); // ADC
  t1->SetBranchAddress("adc3",   &adc3_1); // ADC
  t1->SetBranchAddress("adc4",   &adc4_1); // ADC
  Double_t ent1 = t1->GetEntries(); // Number of events
  for(int i=0 ; i<ent1 ; i++){
    t1->GetEntry(i);
    // ~~~ Cut conditions ~~~~
    if(40.0<time1_1  && time1_1<100.0 && 
       1700.0<adc1_1 && adc1_1<3500.0 && 
       40.0<time2_1  && time2_1<100.0 && 
       1000.0<adc2_1 && adc2_1<3500.0 &&
       40.0<time3_1  && time3_1<100.0 && 
       1000.0<adc3_1 && adc3_1<3500.0 &&
       40.0<time4_1  && time4_1<100.0 && 
       1000.0<adc4_1 && adc4_1<3500.0){
      cut = true;
    }
    else cut = false;

    if(cut==true){
      h_center->Fill(time2_1-time1_1-timeoffset);
      //add34 = add34 + (time2-time1); // ?
    }
  }
  // ~~~~ Scaled histogram ~~~~
  max1 = h_center->GetMaximum();
  //h_center -> Scale(1.0/max1);
  Double_t val,val_er;
  for(int i=0 ; i<bin1 ; i++){
    val = h_center->GetBinContent(i+1);
    val_er = sqrt(val);
    val = val/max1;
    val_er = val_er/max1;
    h_center_scaled->SetBinContent(i+1,val);
    h_center_scaled->SetBinError(i+1,val_er);
  }
  
  // ======== LOOP for "f2" ==================
  t2->SetBranchAddress("time1",  &time1_2); // time = tdc * 0.05 [ns]
  t2->SetBranchAddress("time2",  &time2_2); // time = tdc * 0.05 [ns]
  t2->SetBranchAddress("time3",  &time3_2); // time = tdc * 0.05 [ns]
  t2->SetBranchAddress("time4",  &time4_2); // time = tdc * 0.05 [ns]
  t2->SetBranchAddress("adc1",   &adc1_2); // ADC
  t2->SetBranchAddress("adc2",   &adc2_2); // ADC
  t2->SetBranchAddress("adc3",   &adc3_2); // ADC
  t2->SetBranchAddress("adc4",   &adc4_2); // ADC
  Double_t ent2 = t2->GetEntries(); // Number of events
  for(int i=0 ; i<ent2 ; i++){
    t2->GetEntry(i);
    // ~~~ Cut conditions ~~~~
    if(40.0<time1_2  && time1_2<100.0 && 
       1700.0<adc1_2 && adc1_2<3500.0 && 
       40.0<time2_2  && time2_2<100.0 && 
       1000.0<adc2_2 && adc2_2<3500.0 &&
       40.0<time3_2  && time3_2<100.0 && 
       1000.0<adc3_2 && adc3_2<3500.0 &&
       40.0<time4_2  && time4_2<100.0 && 
       1000.0<adc4_2 && adc4_2<3500.0){
      cut = true;
    }
    else cut = false;

    if(cut==true){
      h_plus->Fill(time2_2-time1_2-timeoffset);
    }
  }
  // ~~~~ Scaled histogram ~~~~
  max2 = h_plus->GetMaximum();
  for(int i=0 ; i<bin1 ; i++){
    val = h_plus->GetBinContent(i+1);
    val_er = sqrt(val);
    val = val/max2;
    val_er = val_er/max2;
    h_plus_scaled->SetBinContent(i+1,val);
    h_plus_scaled->SetBinError(i+1,val_er);
  }
  
  // ====== Draw histograms =================
  TCanvas* c1 = new TCanvas("c1","c1");
  gPad->SetGridx();gPad->SetGridy();
  h_center_scaled->Draw();
  h_plus_scaled->Draw("same");
  
  // =========== Fitting histograms =============
  Double_t fitmin = -0.8 , fitmax = 0.8;
  Double_t space = 1.5;
  // ~~~ Fitting for data of center crossing ~~~~
  TF1* func1 = new TF1("func1","[0]/sqrt(2.0*3.14159)/[2] * exp(-0.5*pow((x-[1])/[2],2.0))",-4.0,4.0);
  func1->SetParameters(1.0,0.0,0.45);
  func1->SetParLimits(0,0.80,1.20);
  func1->SetParLimits(3,0.35,0.50);
  h_center_scaled->Fit("func1","NB","",fitmin,fitmax);
  func1->SetLineColor(1);
  func1->Draw("same");
  
  // ~~~ Fitting for data of +11.5 cm crossing ~~~~
  TF1* func2 = (TF1*)func1->Clone("func2");
  func2->SetParameters(1.0,1.5,0.45);
  func2->SetParLimits(0,0.80,1.20);
  func2->SetParLimits(3,0.35,0.50);
  h_plus_scaled->Fit("func2","NB","",fitmin+space,fitmax+space);
  func2->SetLineColor(2);
  func2->Draw("same");
  
  // ======= Results =========
  Double_t mean1    = 0.0 , mean2    = 0.0;
  Double_t mean1_er = 0.0 , mean2_er = 0.0;
  Double_t time_diff    = 0.0;
  Double_t time_diff_er = 0.0;
  Double_t sol    = 0.0; // Speed Of Light
  Double_t sol_er = 0.0; 
  const double length1 = 0.115; // Moved by 11.5 cm from the center
  
  mean1 = func1->GetParameter(1);
  mean1_er = func1->GetParError(1);
  mean2 = func2->GetParameter(1);
  mean2_er = func2->GetParError(1);
  time_diff = mean2 - mean1;
  time_diff_er = sqrt(pow(mean1_er,2.0) + pow(mean2_er,2.0));
  sol = length1*2.0/(time_diff*1.0e-9);
  sol_er = sol * time_diff_er/time_diff;
  
  cout << endl;
  cout << " Time difference:    " << time_diff << " +/- " << time_diff_er << " ns" << endl;
  cout << " --> Speed of light: " << sol << " +/- " << sol_er << " m/s" << endl;
  
  // =========== Create graphs ===============
  // Also, you can obtaing the speed of light by fitting 
  // to a graph which will be made here.
  const int n=2;
  Double_t x[n]   = {0.0 , 23.0};
  Double_t xer[n] = {0.0 , 0.0};
  Double_t y[n]   = {mean1, mean2};
  Double_t yer[n] = {mean1_er, mean2_er};
  TGraphErrors* gr_dragon = new TGraphErrors(n,y,x,yer,xer);
  gr_dragon->SetName("gr_dragon");
  // ~~~ Empty histogram ~~~
  TH2F* h_emp = new TH2F("h_emp","",100,-3.0,3.0,100,-30.0,30.0);
  h_emp->GetXaxis()->SetTitle("[ns]");
  h_emp->GetYaxis()->SetTitle("[cm]");
  // ~~~ Draw the graph ~~~
  TCanvas* c2 = new TCanvas("c2","c2");
  gPad->SetGridx();gPad->SetGridy();
  h_emp->Draw();
  gr_dragon->Draw("*samel");
  gr_dragon->SetMarkerStyle(25);
  
  // ===== Create new ROOT file ======
  TFile* fnew = new TFile("spl_cosmic_results.root","recreate");
  TObjArray hlist(0);
  hlist.Add(c1);
  hlist.Add(c2);
  hlist.Add(h_center);
  hlist.Add(h_plus);
  hlist.Add(h_center_scaled);
  hlist.Add(h_plus_scaled);
  hlist.Add(func1);
  hlist.Add(func2);
  hlist.Add(gr_dragon);
  hlist.Write();
  fnew->Close();
  cout << endl;
  cout << "  " << fnew->GetName() << " was created for check." << endl;
  
  // ========= Print =============
  //c1->Print("spl_cosmic_20141113_1.eps","eps");
  //c2->Print("spl_cosmic_20141113_2.eps","eps");
  
    
}

/*
Processing spl_cosmic.cc...
 FCN=17.0193 FROM MIGRAD    STATUS=CONVERGED      79 CALLS          80 TOTAL
                     EDM=8.30112e-08    STRATEGY= 1      ERROR MATRIX ACCURATE 
  EXT PARAMETER                                   STEP         FIRST   
  NO.   NAME      VALUE            ERROR          SIZE      DERIVATIVE 
   1  p0           1.16266e+00   1.21701e-02   1.98685e-04   2.92900e-03
   2  p1           3.05133e-02   5.37927e-03   1.10893e-05   3.06297e-02
   3  p2           4.56973e-01   5.84048e-03   1.10458e-05  -6.51062e-02
 FCN=123.146 FROM MIGRAD    STATUS=CONVERGED      75 CALLS          76 TOTAL
                     EDM=5.50558e-07    STRATEGY= 1      ERROR MATRIX ACCURATE 
  EXT PARAMETER                                   STEP         FIRST   
  NO.   NAME      VALUE            ERROR          SIZE      DERIVATIVE 
   1  p0           1.05173e+00   4.05030e-03   1.04422e-04   4.55860e-02
   2  p1           1.34030e+00   1.99583e-03   9.68220e-06  -2.55519e-01
   3  p2           4.26140e-01   1.88134e-03   8.75228e-06  -2.55735e-01

 Time difference:    1.30979 +/- 0.00573759 ns
 --> Speed of light: 1.75601e+08 +/- 769225 m/s

*/
